#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "install.h"


bool
InstallDriver(
    IN SC_HANDLE  SchSCManager,
    IN LPCTSTR    DriverName,
    IN LPCTSTR    ServiceExe
    )
{
    SC_HANDLE   schService;
    DWORD       err;

    //
    // NOTE: This creates an entry for a standalone driver. If this
    //       is modified for use with a driver that requires a Tag,
    //       Group, and/or Dependencies, it may be necessary to
    //       query the registry for existing driver information
    //       (in order to determine a unique Tag, etc.).
    //

    //
    // Create a new a service object.
    //
    schService = CreateService(SchSCManager,           // handle of service control manager database
                               DriverName,             // address of name of service to start
                               DriverName,             // address of display name
                               SERVICE_ALL_ACCESS,     // type of access to service
                               SERVICE_KERNEL_DRIVER,  // type of service
                               SERVICE_DEMAND_START,   // when to start service
                               SERVICE_ERROR_NORMAL,   // severity if service fails to start
                               ServiceExe,             // address of name of binary file
                               NULL,                   // service does not belong to a group
                               NULL,                   // no tag requested
                               NULL,                   // no dependency names
                               NULL,                   // use LocalSystem account
                               NULL                    // no password for service account
                               );

    if (schService == NULL) 
	{
        err = GetLastError();

        if (err == ERROR_SERVICE_EXISTS) 
		{
            // Ignore this error.
            return true;
        } 
		else 
		{
            //printf("CreateService failed!  Error = %d \n", err );
            // Indicate an error.
            return false;
        }
    }

    // Close the service object.
    if (schService) 
	{
        CloseServiceHandle(schService);
    }

    // Indicate success.
    return true;
                              
}  


bool
ManageDriver(
    IN LPCTSTR  DriverName,
    IN LPCTSTR  ServiceName,
    IN USHORT   Function
    )
{

    SC_HANDLE   schSCManager;
    bool rCode = TRUE;

    // Insure (somewhat) that the driver and service names are valid.
    if (!DriverName || !ServiceName) 
	{
        //printf("Invalid Driver or Service provided to ManageDriver() \n");
        return false;
    }

    // Connect to the Service Control Manager and open the Services database.
    schSCManager = OpenSCManager(NULL,                   // local machine
                                 NULL,                   // local database
                                 SC_MANAGER_ALL_ACCESS   // access required
                                 );

    if (!schSCManager) 
	{
        //printf("Open SC Manager failed! Error = %d \n", GetLastError());
        return false;
    }

    // Do the requested function.
    switch( Function ) 
	{
        case DRIVER_FUNC_INSTALL:
            // Install the driver service.
            if (InstallDriver(schSCManager,
                              DriverName,
                              ServiceName
                              )) 
			{
                // Start the driver service (i.e. start the driver).
                rCode = StartDriver(schSCManager,
                                    DriverName
                                    );
            } 
			else 
			{
                // Indicate an error.
                rCode = false;
            }
            break;

        case DRIVER_FUNC_REMOVE:
            // Stop the driver.
            StopDriver(schSCManager,
                       DriverName
                       );

            // Remove the driver service.
            RemoveDriver(schSCManager,
                         DriverName
                         );

            // Ignore all errors.
            rCode = true;
            break;

        default:
            printf("Unknown ManageDriver() function. \n");
            rCode = false;
            break;
    }

    // Close handle to service control manager.
    if (schSCManager) 
	{
        CloseServiceHandle(schSCManager);
    }

    return rCode;
}   // ManageDriver


bool
RemoveDriver(
    IN SC_HANDLE    SchSCManager,
    IN LPCTSTR      DriverName
    )
{
    SC_HANDLE   schService;
    bool     rCode;

    // Open the handle to the existing service.
    schService = OpenService(SchSCManager,
                             DriverName,
                             SERVICE_ALL_ACCESS
                             );

    if (schService == NULL) 
	{
        //printf("OpenService failed!  Error = %d \n", GetLastError());
        // Indicate error.
        return false;
    }

    // Mark the service for deletion from the service control manager database.
    if (DeleteService(schService)) 
	{
        // Indicate success.
        rCode = true;
    } 
	else 
	{
        //printf("DeleteService failed!  Error = %d \n", GetLastError());
		// Indicate failure.  Fall through to properly close the service handle.
        rCode = false;
    }

    // Close the service object.
    if (schService) 
	{
        CloseServiceHandle(schService);
    }
    return rCode;
}   


bool
StartDriver(
    IN SC_HANDLE    SchSCManager,
    IN LPCTSTR      DriverName
    )
{
    SC_HANDLE   schService;
    DWORD       err;

    // Open the handle to the existing service.
    schService = OpenService(SchSCManager,
                             DriverName,
                             SERVICE_ALL_ACCESS
                             );

    if (schService == NULL) 
	{
        //printf("OpenService failed!  Error = %d \n", GetLastError());
        // Indicate failure.
        return false;
    }

    // Start the execution of the service (i.e. start the driver).
    if (!StartService(schService,     // service identifier
                      0,              // number of arguments
                      NULL            // pointer to arguments
                      )) 
	{
        err = GetLastError();
        if (err == ERROR_SERVICE_ALREADY_RUNNING) 
		{
            // Ignore this error.
            return true;
        } 
		else 
		{
            //printf("StartService failure! Error = %d \n", err );
            // Indicate failure.  Fall through to properly close the service handle.
            return false;
        }
    }

    // Close the service object.
    if (schService) 
	{
        CloseServiceHandle(schService);
    }
    return true;
}   // StartDriver


bool
StopDriver(
    IN SC_HANDLE    SchSCManager,
    IN LPCTSTR      DriverName
    )
{
    bool         rCode = TRUE;
    SC_HANDLE       schService;
    SERVICE_STATUS  serviceStatus;

    // Open the handle to the existing service.
    schService = OpenService(SchSCManager,
                             DriverName,
                             SERVICE_ALL_ACCESS
                             );

    if (schService == NULL) 
	{
        printf("OpenService failed!  Error = %d \n", GetLastError());
        return false;
    }

    // Request that the service stop.
    if (ControlService(schService,
                       SERVICE_CONTROL_STOP,
                       &serviceStatus
                       )) 
	{
        // Indicate success.
        rCode = true;
    } 
	else 
	{
        printf("ControlService failed!  Error = %d \n", GetLastError() );
        // Indicate failure.  Fall through to properly close the service handle.
        rCode = false;
    }

    // Close the service object.
    if (schService) 
	{
        CloseServiceHandle (schService);
    }
    return rCode;
}   //  StopDriver



